/**
 * Copyright 2014 (c) Anna Schumaker.
 */
extern "C" {
#include <core/string.h>
}
#include <core/tags/generic.h>

GenericTag :: GenericTag() {}

GenericTag :: GenericTag(const std::string &name)
	    : _name(name)
{
	gchar *g_lc = string_lowercase(name.c_str());
	_lower = g_lc;
	g_free(g_lc);
}

GenericTag :: GenericTag(const GenericTag &tag)
	    : _name(tag._name), _lower(tag._lower)
{
}

const std::string GenericTag :: primary_key() const
{
	return _name;
}

void GenericTag :: read(file &file)
{
	gchar *name = file_readl(&file);
	gchar *g_lc = string_lowercase(name);

	_name  = name;
	_lower = g_lc;

	g_free(name);
	g_free(g_lc);
}

void GenericTag :: write(file &file)
{
	file_writef(&file, "%s", _name.c_str());
}

const std::string &GenericTag :: name() const
{
	return _name;
}

const std::string &GenericTag :: lowercase()
{
	return _lower;
}

int GenericTag :: compare(const GenericTag *rhs)
{
	return string_compare(_lower.c_str(), rhs->_lower.c_str());
}
